//
//      Copyright Timothy Evans 2014    timothy.evans@gsecraif.org
//
//      This file is part of gsecraif.
//
//      gsecraif is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 3 of the License, or
//      (at your option) any later version.
//
//      See gsecraif.cpp
//
// =====================================================================
//
// parse command line arguments
//
// $Date$
// $Revision$
// =====================================================================
// 
// Compile with g++
#include "parse_args.h"

using namespace std;

extern unsigned short int debuglevel;
extern bool showhashes;


void parse_args(int argc, char *argv[], char &action, int &numoffiles,
                unsigned short int &leftrotate, vector<string> &compfilenames)            
{
const short int maxnumoffiles = 255;    // maximum number of files programme can split in to
short int argcn = 1;                    // used to parse args
short int suppliedfilename = 1;         // used to access each supplied file name
string postfix = "000";                 // 3 character postfix
const string defprefix = "split";       // default prefix string
string prefixstring;                    // string to hold supplied prefix
// note that if the prefixstring is assigned the defprefix then the length
// of prefixstring is limited to the length of defprefix when argv is assigned
unsigned short int prefixarg=0;         // which argument is the prefix.
string tempstring;                      // temporary string to hold constructed file name

unsigned short int filenamesstart = 1 ;
        // names of component files can be given on the command line
        // - this is where they start

char whatcompfilenames = 'd';          // what is the format of the component filenames,
                                       // one of:
                                       // d default prefix
                                       // p prefix supplied
                                       // s component file names
                                       // supplied on command line

vector<string>::iterator compfileid;
	
if(argc>1) {
  
    while (argcn < argc) {
        if ( (!strcmp(argv[argcn], "-h")) || (!strcmp(argv[argcn], "--help")) ) {
            action = 'h';
            filenamesstart++;	// anticipate checks at end of function
            break;
            }
            
        else if ( (!strcmp(argv[argcn], "-V")) || (!strcmp(argv[argcn], "--version")) ) {
            action = 'v';
            filenamesstart++;	// anticipate checks at end of function 
            break;
            }
                 
        else if ( (!strcmp(argv[argcn], "-c")) || (!strcmp(argv[argcn], "--combine")) ) {
            action = 'c';
            }
            
        else if ( (!strcmp(argv[argcn], "-n")) || (!strcmp(argv[argcn], "--number")) ) {
            if (argcn<(argc - 1)) {
                ++argcn;
                ++filenamesstart;
                numoffiles = atoi(argv[argcn]); 
                if ((numoffiles > maxnumoffiles) || (numoffiles < 3)) {
                    action = 'u';
                    }
                
                } // if (argcn<(argc - 1...
            }
            
       else if ( (!strcmp(argv[argcn], "-r")) || (!strcmp(argv[argcn], "--rotate")) ) {
           if (argcn<(argc - 1)) {
                ++argcn;
                ++filenamesstart;
                leftrotate = atoi(argv[argcn]); 
                if ((leftrotate > 7) || (leftrotate < 0)) {
                    action = 'u';
                    }
                } 
            }
            
        else if ((!strcmp(argv[argcn], "-d")) || (!strcmp(argv[argcn], "--debug")) ) {
            if (argcn<(argc - 1)) {
                ++argcn;
                ++filenamesstart;
                debuglevel = atoi(argv[argcn]); 
                if ((debuglevel > 2) || (debuglevel < 0)) {
                    action = 'u';
                    }
                } 
            }
            
        else if ( (!strcmp(argv[argcn], "-p")) || (!strcmp(argv[argcn], "--prefix")) ) {
            if (argcn<(argc - 1)) {
                whatcompfilenames = 'p';
                ++argcn;
                ++filenamesstart;
                prefixarg=argcn;
                prefixstring = argv[prefixarg];
                } 
            }
            
        else if ( (!strcmp(argv[argcn], "-#")) || (!strcmp(argv[argcn], "--hash")) ) {
            showhashes = true; 
            }
            
        else if (strncmp(argv[argcn], "-",1) !=0) {
            // not a parameter, component file list begins here
            break;
            }
               
        else {
            action='u';
            break;
            }
            
    ++argcn;
    ++filenamesstart;
    } // while (argcn

} // if(argc

// if the value of filenamestart is greater than the position of the last argument
// then no file names were supplied
// note zeroth argument is program name
// note for options such as -V or -h filenamestart is increased to make the logic below simpler -
// simply test if filenamestart < argc rather than also check for -V or -h
if (filenamesstart < argc) {
    // file names were supplied, check there are at least three file names
    if ((argc - filenamesstart) < 3) action='u';
    // check that a prefix was not supplied with some files names
    if (whatcompfilenames == 'p') action = 'u';
    numoffiles = argc - filenamesstart;
    whatcompfilenames = 's';  	
    }


// build the list of component file names:
suppliedfilename = filenamesstart;
for (unsigned int compfileid=0; compfileid <= (numoffiles - 1); ++compfileid) {
    if (whatcompfilenames != 's') { 
        int2asciistr(postfix, compfileid);
        if (whatcompfilenames == 'p') {
            tempstring = prefixstring + postfix;
            compfilenames.push_back(tempstring);
            }
        else {
            prefixstring = defprefix;               // assign default
            tempstring = prefixstring + postfix;
            compfilenames.push_back(tempstring);
            } 
        } // if (whatcompfilenames... 
    else if (whatcompfilenames != 'u') {
        // component filenames have been supplied on the command line
        tempstring = argv[suppliedfilename];
        compfilenames.push_back(tempstring);
        ++suppliedfilename;
        }
        
} // for (compfileid=0..


// Print some basic information if debug level > 0
if (debuglevel) {
    if (action == 'c') {
        cerr << "Combining files and output to standard out. \n";
        }
    else {
        cerr << "Splitting from standard in. \n";
        }
    cerr << "Number of files: " << numoffiles << "\n";
    cerr << "Rotation: " << leftrotate << "\n";
    cerr << "File names start at: " << filenamesstart << "\n";
    while (argcn < argc) {
        cerr << argv[argcn] << ", ";    
        ++argcn;
        }
        
    cerr << "\n";
    cerr << "Debug level: " << debuglevel << "\n----------\n";
    } // if (debug...

// -----

} // end of function
